/*  $Id: dion_server.pl,  v0.8.1 2006/2/22  zhisheng Exp $


    Author:        Zhisheng Huang
    E-mail:        huang@cs.vu.nl
    WWW:           http://www.cs.vu.nl/~huang
    Copyright (C): 2005-2006 Vrije University Amsterdam

This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    As a special exception, if you link this library with other files,
    compiled with a Free Software compiler, to produce an executable, this
    library does not by itself cause the resulting executable to be covered
    by the GNU General Public License. This exception does not however
    invalidate any other reasons why the executable file might be covered by
    the GNU General Public License.

*/

% this server requires XDIG version 1.5.0 or higher.

user:file_search_path(xdig, '../xdig').

:-use_module(xdig('dig_client')).
:-use_module(xdig('dig_server')).
:-use_module(xdig('dig_process')).
:-use_module(xdig('dig_db')).

:-use_module(xdig('pion/pion_selection')).
:-use_module(xdig('owl/owl2dig')).

:-use_module(library('http/thread_httpd')).





%deal with the tell request
my_dig_server_processing(RequestData, Answer, [connection(close)]):-
			dig_requestdata_analysis(RequestData, Data, Type),
			Type=tells,
			!,
%			dig_post(RequestData, Answer, [connection(close)]),
                  dion_setting(kb, KBID),
                  dig_assert_data(KBID, Data),
                  dig_standard_response(ok, _ID, Answer).



%deal with the ask request
my_dig_server_processing(RequestData, Answer, _Options):-
			dig_requestdata_analysis(RequestData, Data, Type),
			Type=asks,
			!,
			dig_post(RequestData, Answer1, [connection(close)]),
			pion_answer_preprocessing(Answer1, Data, Answer),
                  true.


%deal with owl tell request
my_dig_server_processing(RequestData, Answer, [connection(close)]):-
			dig_requestdata_analysis(RequestData, _Data, Type),
			Type = 'rdf:RDF',
                  !,
               	owl2dig(RequestData, elements(L)),
			dig_post(elements(L), Answer, [connection(close)]),
                  dig_get_element(L, E, _),
                  E = element(tells, _, L1),
                 	dion_setting(kb, KBID),
 		     	dig_assert_data(KBID, L1),
%%%               retractall(assert(dig_db_element(owl(dion, KBID), _))),
%%%               assert(dig_db_element(owl(dion, KBID), status(yes))),
%%%               assert(dig_db_element(owl(dion, KBID), RequestData)),	
			true.


%deal with the set request
my_dig_server_processing(RequestData, Answer, _Options):-
			dig_requestdata_analysis(RequestData, Data, Type),
			Type = set,
			!,
			dig_setting_processing(Data, A),
                  Reply = [element(response, [], A)],
                  Answer = answer(_, Reply),
                  true.

%deal with the do operation

my_dig_server_processing(RequestData, Answer, _Options):-
			dig_requestdata_analysis(RequestData, Data, Type),
			Type = do,
			!,
                  dion_setting(localDoPredicates, L),
 %                 format(user_error, 'L=~w~n', [L]),
                  get_predicate(Data, Predicate, Parameters),
 %                 format(user_error, 'Predicate=~w~n', [Predicate]),
			(member(Predicate, L)->dion_do_processing(Predicate, Parameters, A);dig_do_processing(Data, A)),
                  Reply = [element(response, [], A)],
                  Answer = answer(_, Reply),
                  true.



%deal with other request
 my_dig_server_processing(RequestData, Answer, Options):-
			dig_post(RequestData, Answer, Options).



get_predicate(Data, Predicate, Parameters):-
       E = element(function, [name=Predicate], []),
       dig_get_element(Data, E, Parameters),
       true.



%mups_mips_core_pinpoint(+Ontology, +MUPSNonAxiomForm, -PinpointAxiomForm)

mups_mips_core_pinpoint(Ontology, ML, P):-
		mips_to_file(Ontology, ML, M1),
            core_to_file(Ontology, M1, C),
		pinpoint_to_file(Ontology, C, M1, P),
            true.

%mips_to_file(+Ontology, +MUPSNonAxiomForm, -MIPSAxiomForm)

mips_to_file(Ontology, ML, M1):-
            mips(ML, M),
            mips_axiom(M, M1),            
            dion_setting(pinpointDirectory, D),
            concat_atom([D, '/mips_', Ontology, '.txt'], File2),           
    		open(File2, write, Out2, []),
 %           write(Out2, M1), 
            format(Out2, '~q.~n', [M1]), 
            close(Out2),
		true.

%core_to_file(+Ontology, +MIPSAxiomForm, -CoreAxiomForm):-

core_to_file(Ontology, M1, C):-
            mips_core(M1, C),
            dion_setting(pinpointDirectory, D),
            concat_atom([D, '/core_', Ontology, '.txt'], File3),           
    		open(File3, write, Out3, []),
            format(Out3, '~q.~n', [C]), 
%            write(Out3, C), 
            close(Out3),
            true.

%pinpoint_to_file(+Ontology, +CoreAxiomForm, +MIPSAxiomForm, -PinpointAxiomForm):-

pinpoint_to_file(Ontology, C, M1, P):-
            pinpoints(C, M1, P),
            dion_setting(pinpointDirectory, D),
		concat_atom([D, '/pinpoint_', Ontology, '.txt'], File4),           
    		open(File4, write, Out4, []),
%            write(Out4, P), 
            format(Out4, '~q.~n', [P]), 
            close(Out4),
		true.



%unsatisfiable_concept_list_to_file(+Ontology, +KB, -L, -CL, -N, -UCText)
% create an unsatisfiable concept file for Ontology
% the file Ontology.xml should be available for the data input

unsatisfiable_concept_list_to_file(Ontology, KB,  L, CL, N, UCText):-
            concat_atom([Ontology, '.xml'], File0),           
		dig_load_telldata(File0, KB),
            dig_unsatisfiable_conceptname(KB, L),
            pion_icr_set(L, icr(_, CL, _)),
            length(CL, N),
            E = element(unsatisfiableConcept, [ontology=Ontology, number=N], L),
            elements_xmltext_text([E], UCText),           
            dion_setting(pinpointDirectory, D),
            concat_atom([D, '/usc_', Ontology, '.xml'], File1),           
    		open(File1, write, Out1, []),
            write(Out1, UCText), 
            close(Out1),
            true.


% get_mupss_from_file(+UnsatisfiableConceptSet, -MUPS)
% get MUPS from the mups files for a set of Concept

get_mupss_from_file([], []):-!.

get_mupss_from_file([C|Y], ME):-
      get_mups_from_file(C, L, _N),
      !,
      get_mupss_from_file(Y, ME1),
      append(L, ME1, ME).


% get_mups_from_file(+UnsatisfiableConcept, -MUPS, -NumberOfMUPS)
% get MUPS from the mups files for a Concept

get_mups_from_file(C, L, N):-
            dion_setting(pinpointFileDirectory, D),
		get_shorted_concept_name(C, C1),
		concat_atom([D, '/mups_', C1, '.xml'], File),           
 		open(File, read, In),
	        load_structure(In, Elements, []),
                close(In), 
 		dig_get_element(Elements, E, _),
                E = element('MUPSs', [concept=C, number=N], L1),           
                mups_mupss(L1, L),   
                true.


%get_unsatisfiable_concept_from_file(+Ontology, -UnsatisfiableConceptSet, -NumberOfUnsatisfiableConcept)
% 

get_unsatisfiable_concept_from_file(Ontology, L, N):-
                dion_setting(pinpointFileDirectory, D),
		concat_atom([D, '/usc_', Ontology, '.xml'], File1),           
 		open(File1, read, In),
	        load_structure(In, Elements, []),
                close(In), 
		dig_get_element(Elements, E, _),
                E = element(unsatisfiableConcept, [ontology=Ontology, number=N], L1),           
                pion_icr_set(L1, icr(_, L, _)),
                true.


%get_disjoint_statements(+ConceptChildrenList, -ChildfenDisjointList)


get_disjoint_statements([],[]):-!.

get_disjoint_statements([E|Y],Y2):-
  E = children(_C, L),
  element_number(L, N),
  N > 1, 
  !,
  E1 = element(disjoint, [], L),
  get_disjoint_statements(Y, Y1),
  Y2 = [E1|Y1].

get_disjoint_statements([_E|Y],Y1):-
  !,
  get_disjoint_statements(Y, Y1).

 
% element_number(+List, -NumberOfElement)

element_number([], 0):-!.

element_number([X|Y], N):-
    X = element(_,_,_),
    !,
    element_number(Y, N1),
    N is N1+1.

element_number([_X|Y], N):-
    !,
    element_number(Y, N).
   

%dig_all_concept_children_askelement(+ConceptList, -ConceptChildrenQueryList)

dig_all_concept_children_askelement([E|Y], L7):-
    E = element(catom, [name=C], []),
    E1 = element(children, [id=C], [E]),
    !,  
    dig_all_concept_children_askelement(Y, L6),
    L7 =[E1|L6],
    true.

dig_all_concept_children_askelement([_E|Y], L):-
    !,
   dig_all_concept_children_askelement(Y, L).


dig_all_concept_children_askelement([], []):-!.


%dig_all_concept_children(+ConceptList, -ListOf children(Concept, ChildrenConcepts))

dig_all_concept_children(L0, L7):-
    dig_all_concept_children_askelement(L0, L1),
    dig_ask(elements(L1), Reply, [connection(close)]),
    Reply = answer(_, L2),
    L2 = [element(_,_, L3)],
    get_all_concept_children(L3, L7),
    true.

% get_all_concept_children(+ConceptSet, -ListOf children(Concept, ChildrenConcepts))
% where ChildrenConcept does not include the bottom

get_all_concept_children(L, L5):-
    E = element(conceptSet, [id=C], L1),
    select(E, L, L2),
    !,
    allconceptname(L1, L3),
    (member(element(bottom, [],[]), L3)-> E1 =''; E1 = children(C, L3)),
    get_all_concept_children(L2, L4),
    L5 = [E1|L4].

get_all_concept_children(_L, L5):-!,
    L5 = [].


%mips(+MUPS, -MIPS)

mips([], []):-!.

mips([ME|ML1], MIPS1):-
   superset_of_one_element(ME, ML1),
   !,
   mips(ML1, MIPS1).

mips([ME|ML1], [ME|MIPS1]):-
   !,
   mips(ML1, MIPS1).

% superset_of_one_element(+S, +L)
% The set S is a superset of an element in L
   
superset_of_one_element(ME, [S|_L]):-
    subset(S, ME).


superset_of_one_element(ME, [_S|L]):-
    superset_of_one_element(ME, L).


%mips_axiom(+MIPS, -MIPSwithShortedFormat)
%concerting mips into one with axiom name format
% for a display.

mips_axiom([], []):-!.

mips_axiom([E|L], [A|AL]):-
   mips_axiom_element(E, A),
   !,
   mips_axiom(L, AL).

%mips_axiom_element(+aMIPS, -MIPSwithShortedFormat)

mips_axiom_element([], []):-!.

mips_axiom_element([H|T],[axiom(C)|T1]):-
   H = element(_,_,_),
   get_axiom_label(H, C),
   !,
   mips_axiom_element(T, T1).

mips_axiom_element([_H|L],T):-
   !,
   mips_axiom_element(L, T).


% get_axiom_label(+Element, -AxiomLabel)
% use the axiom label to denote the axiom

get_axiom_label(H, Label):-
   dig_db_element(label(dion, Label), H),
   !.


get_axiom_label(H, C1):-
   H= element(Type, [], L),
   (Type = equalc;  Type=disjoint),
   !,
   select(element(catom,[name=C],[]), L, _),
   concat_atom([C, '_', Type], C1),
   assert(dig_db_element(label(dion, C1), H)),
   true.

get_axiom_label(H, C1):-
   H= element(impliesc, [], L),
   select(element(catom,[name=C],[]), L, L1),
   get_axiom_partial_name(L1, C2),
   concat_atom([C, '_', C2, '_', impliesc], C1),
   assert(dig_db_element(label(dion, C1), H)),
   true.


get_axiom_label(H, C2):-
   get_axiom_partial_name([H], C2),
   !,
   assert(dig_db_element(label(dion, C2), H)).

get_axiom_partial_name([], ''):-!.

get_axiom_partial_name([H|L], N):-
   H = element(Type, [name=C], _),
   (Type = catom; Type=ratom; Type=individual),
   !,
   get_axiom_partial_name(L, N1),
   concat_atom([C, '_', N1], N).


get_axiom_partial_name([H|L], N):-
   H = element(TYPE, _, L2),
   !,
   get_axiom_partial_name(L2, N2),
   get_axiom_partial_name(L, N1),
   concat_atom([TYPE, '_', N2, '_', N1], N).

get_axiom_partial_name([_H|L], N):-
   !,
   get_axiom_partial_name(L, N).



%mips_core(+MIPSwithShortedFormat, -Core)
   
mips_core(L, C):-
   flatten(L, L1),
   get_core(L1, C).

%get_core(+FlattenMIPSwithShortedFormat, -Core)

get_core([], []):-!.

get_core([E|L], C):-
    member(E, L),
    !,
    get_core(L, C1),
    select(core(E, N), C1, C2),
    N1 is N+1,
    C = [core(E, N1)|C2].
    


get_core([E|L], C):-
    \+ member(E, L),
    !,
    get_core(L, C1),
    C = [core(E,1)|C1].

%core_maximal_arity(+Core, -MaximalArity)

core_maximal_arity([], N, N):-!.

core_maximal_arity([X|Y], N1, N):-
    X = core(_E, N2),
    N2 > N1,
    !,
    core_maximal_arity(Y, N2, N).

core_maximal_arity([_X|Y], N1, N):-
    !,
    core_maximal_arity(Y, N1, N).


%pinpoints(+Core, +MIPSwithShortedFormat, -Pinpoints)
% to find all pinpoints

pinpoints(Core, Mips, Pinpoints):-
    setof(X, (pinpoint(Core, Mips, X)), Pinpoints).




%pinpoint(+Core, +MIPSwithShortedFormat, -Pinpoint)
% to find a pinpoint

pinpoint(_Core, [], []):-!.

pinpoint([], _,  []):-!.


pinpoint(Core, Mips, Pinpoint):-
   core_maximal_arity(Core, 0, N),
   select(core(A,N), Core, Core1),
   delete_axiom_from_mips(A, Mips, Mips1, S),
   var(S),
   !,
   pinpoint(Core1, Mips1, Pinpoint1),
   Pinpoint = Pinpoint1.



pinpoint(Core, Mips, Pinpoint):-
   core_maximal_arity(Core, 0, N),
   select(core(A,N), Core, Core1),
   delete_axiom_from_mips(A, Mips, Mips1, S),
   S = 1,
   pinpoint(Core1, Mips1, Pinpoint1),
   Pinpoint = [A|Pinpoint1].



%delete_axiom_from_mips(+Axiom, +Mips, -Mips, -DeletedStatus):-!.
% DeleteStatus = 1: some axiom has been really deleted from +Mips

delete_axiom_from_mips(_A, [], [], _):-!.

delete_axiom_from_mips(A, [X|Y], L, 1):-
   member(A, X),
   !,
   delete_axiom_from_mips(A, Y, L, _).

delete_axiom_from_mips(A, [X|Y], [X|L], S):-   
   delete_axiom_from_mips(A, Y, L, S).




    

    
                
%mups(+KBid, +UnsatisfableConceptList, -MUPS)

mups(_KB, [], []):-!.

mups(KB, [E|L], ML):-
    E = element(catom, [name=C], []),
    !,
    get_mups(KB, C, ME),
    mups_log(C, ME),
    mups(KB, L, ML1),
    append(ME, ML1, ML).


mups(KB, [_E|L], ML):-
    !,
    mups(KB, L, ML).

%mups_log(+Concept, +MUPS)
% converting MUPS into the mups xml file for the Concept 

mups_log(C, ME):-
    length(ME, N),
%    format('~w mups for the concept ~w~n', [N, C]),
    mupss_mups(ME, ME1),
    E = element('MUPSs', [concept=C, number=N], ME1),
    elements_xmltext_text([E], MUPSText),
    dion_setting(pinpointDirectory, D),
    get_shorted_concept_name(C, C1),
    concat_atom([D, '/mups_',C1, '.xml'], File),           
    open(File, write, Out, []),
    write(Out, MUPSText),
    close(Out),
    true.


% mupss_mups(+MUPSs, -MUPSsWithMupsMarkup)
% namely, converting each element E into element(mups, [], E)

mupss_mups([], []):-!.

mupss_mups([E|L],[element(mups, [], E)|ME1]):-
    !,
    mupss_mups(L, ME1).

% mups_mupss(+MUPSsWithMupsMarkup, -MUPSs)
% namely, converting each element(mups, [], E) into E.

mups_mupss([], []):-!.

mups_mupss([E|L],[E1|ME1]):-
    E = element(mups, [], E1),
    !,
    mups_mupss(L, ME1).

mups_mupss([_E|L],ME):-
    !,
    mups_mupss(L, ME).





get_mups(KB, C, ME):-
    relevance_expansion(KB, C, L, S, _K),
%    length(S, N),
%    format('getting mups for ~w with the length ~w~n', [C, N]),   
    enumerate_check(C, L, S, [], ME1),
    minimized_check(C, ME1, ME),
    true.


relevance_expansion(KB, C, L, S, K):-
     s(KB, C, 0, L0),
     satisfiable_check(elements(L0), C, Status),
     Status = false,
     !,
     L = [],
     S = [],
     K  = 0.

relevance_expansion(KB, C, L, S, K):-
     s(KB, C, 0, L0),
     pion_icr_set(L0, ICR0),
     bagof(E, (dig_db_element(KB, E),\+ member(E, L0), pion_direct_concept_relevant(E, ICR0)), S11),
     list_to_set(S11, S1),
     pion_icr_set(S1, ICRs1),
     !,
     relevance_expansion(KB, C, 0, L0, ICR0, S1, ICRs1,  L, S, K).






%relevance_expansion(+KB,  +C:Concept, +K: CurrentStep, 
% +L: s(KB, C, K), +ICR of L, +S: NewSelectedSet at K, +ICRs: ICR of S,
% -L1: s(KB,C, K1) , -S1: SelectedSet at K1, -K1: stop at K1)


relevance_expansion(_KB, _C, K, Lk, _ICRk, S, _ICRs,  L1, S1, K1):-
   S = [],
   !,
   L1 = Lk,
   S1 = S,
   K1 = K.


relevance_expansion(_KB, C, K, Lk, _ICRk, S, _ICRs,  L1, S1, K1):-
     union(Lk, S, L),
     satisfiable_check(elements(L), C, Status),
     Status = false,
     !,
     L1 = Lk,
     S1 = S,
     K1 = K.

relevance_expansion(KB, C, K, Lk, ICRk, S, ICRs, L2, S2, K2):-
     bagof(E, (dig_db_element(KB, E),\+ member(E, Lk), pion_direct_concept_relevant(E, ICRk)), S11),
     list_to_set(S11, S1),
     K1 is K+1,
     ICRk= icr(Ik, Ck, Rk),
     ICRs= icr(Is, Cs, Rs),
     union(Lk, S, L),
     union(Ik, Is, I),
     union(Ck, Cs, C1),
     union(Rk, Rs, R),
     ICR = icr(I, C1, R),
     pion_icr_set(S1,  ICRs1),
     relevance_expansion(KB, C, K1, L, ICR, S1, ICRs1, L2, S2, K2).




 
%selection function: s(+KB, +C, +0, -SelectedSet)

s(KB, C, 0, L):- 
 E2 = element(catom, [name=C], []),
 bagof(E, (dig_db_element(KB, E),pion_direct_concept_relevant(E, E2)), L11),
 list_to_set(L11, L),
 true.


% enumerate_check(+Concept, +L: CommonSet, +S:NewSelectedSet, +WorkingMUPS, -MUPS) 
enumerate_check(_C, _L, [], ME, ME1):-
 !,
 ME1 = ME.




enumerate_check(C, L, [E|T], ME, ME1):-
      L1 =[E|L],
	satisfiable_check(elements(L1), C, Status),
      mups_status_check(Status, E, L, ME, L2, ME2),
%%    get_axiom_label(E, LE),
%%    format('Status: ~w for ~w~n', [Status, LE]),
      !,
      enumerate_check(C, L2, T, ME2, ME1).


%mups_status_check(+Status, +Element, +CommonSet, +MUPS, -NewCommonSet, -NewMUPS)

mups_status_check(true, E, L, ME, L2, ME2):-
     !,
     L2= [E|L],
     ME2 =ME.

mups_status_check(false, E, L, ME, L2, ME2):-
     !,
     L2= L,
     append([[E|L]], ME, ME2),
%%   length(ME2, N),
%%   format('new mups has been added, the length of mups list is ~w~n', [N]),
     true.
    
mups_status_check(Status, E, L, ME, L2, ME2):-
    format('unknown status:~w for ~w~n', [Status, E]),
    L2 = L,
    ME2 = ME.


% minimized_check(+Concept, +MUPSs, -MUPSsAfterMinimalCheck):- 

   
 minimized_check(_C, [], []):-!.

 minimized_check(C, [E|L], L1):-
      minimized_check_element(C, E, E1, _),
      !,
      minimized_check(C, L, L2),
      L1 = [E1|L2].


%minimized_check_element(+Concept, +MUPS, -MUPSAfterMinimalCheck, ?FirstElementShouldBeKept)


minimized_check_element(_C, E, E1, First):-
     nonvar(First),
     E = [First|_],
     !,
     E1 = E.

minimized_check_element(C, E, E1, First1):-
     E = [_First|L],
     satisfiable_check(elements(L), C, Status),
     Status = false,
     !,
     minimized_check_element(C, L, E1, First1).


minimized_check_element(C, E, E1, First1):-
     var(First1),
     E = [First|L],
     !,
     append(L, [First], L1),
     minimized_check_element(C, L1, E1, First).

minimized_check_element(C, E, E1, First1):-
     E = [First|L],
     !,
     append(L, [First], L1),
     minimized_check_element(C, L1, E1, First1).


get_pinpoints_from_ontology(Ontology,P):-
            KB = kb(dion,Ontology),
%	    unsatisfiable_concept_list_to_file(Ontology, KB, L, _CL, _N, _UCText), 
            dig_unsatisfiable_conceptname(KB, L),
 %          format(user_error, 'USC:~w~n',[L]),                    
            mups(KB, L, ML),
            mips_axiom(ML, _ML1),
            mips(ML, M),
            mips_axiom(M, M1),
            mips_core(M1, C),
            pinpoints(C, M1, P),
            true.

get_pinpoints_from_kbid(KB,P):-
            dig_unsatisfiable_conceptname(KB, L),                  
            mups(KB, L, ML),
            mips_axiom(ML, _ML1),
            mips(ML, M),
            mips_axiom(M, M1),
            mips_core(M1, C),
            pinpoints(C, M1, P),
            true.



get_pinpoints_from_file(Ontology, P):- 
 		get_unsatisfiable_concept_from_file(Ontology, L, _N),
%            format(user_error, 'Unsatisfiable ~w Concepts:~w~n', [N, L]),
            get_mupss_from_file(L, ML),           
%            format(user_error, 'get mupss ~w~n', [ML]),
            mups_mips_core_pinpoint(Ontology, ML, P),
%            format(user_error, 'pinpoint:~w~n', [P]),
            true.

pinpoints_formatted([], []):-!.

pinpoints_formatted([X|Y], Z):-
    dig_axioms_elements(dion, X, Elements),
    E = element(pinpoint, [], Elements),
    !,
    pinpoints_formatted(Y, Y1),
    Z = [E|Y1].

dig_axioms_elements(_, [], []):-!.
dig_axioms_elements(System, [X|Y], Z):-
  X = axiom(Label),
  dig_db_element(label(System, Label), H), 
  !,
  dig_axioms_elements(System, Y, Y1),
  Z = [H|Y1].

dig_axioms_elements(System, [X|Y], Z):-
  !,
  format(user_error, 'error to get the elements for ~w~n', [X]),
  dig_axioms_elements(System, Y, Z).

    
 
%satisfiable_check(+KB or Elements, +Concept, -Status)

satisfiable_check(kb(KB), C, Status):-
     !,
     Options = [connection(close)],
     dig_db_post(KB, withClearKB, _Answer, Options),
     Element = element(satisfiable, [id=C], [element(catom, [name=C], [])]),
     dig_ask(elements([Element]), Reply, Options),
     Reply = answer(_, L),
     L = [element(_, _, L1)],
     select(element(Status, [id=C], []), L1, _),
     true.

satisfiable_check(elements(L2), C, Status):-
     !,
     Options = [connection(close)],
     L3 = [element(clearKB, [], [])|L2],
%    message_log(L3),
     dig_tell(elements(L3), _Reply, Options),    
     Element = element(satisfiable, [id=C], [element(catom, [name=C], [])]),
     dig_ask(elements([Element]), Reply, Options),
     Reply = answer(_, L),
     L = [element(_, _, L1)],
     select(element(Status, [id=C], []), L1, _),
     true.
         
satisfiable_check(_KB, _C, Status):-
      !,
      Status = error,
      format('never be here for satisfiable check!!!~n', []).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  DION processing predicates, 
%  may be moved to xdig processing library
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


get_shorted_concept_name(C2, C3):-
    sub_string(C2, X, _Y, Z, '#'),
    X>0,
    !,
    X1 is X+1,
    sub_string(C2, X1, Z, _W, C3).


get_shorted_concept_name(C2, C2):-!.




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  DION setting
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

dion_setting(X, Y):-
   dig_db_element(setting(dion, X), Y).

dion_init:-
   retractall(dig_db_element(setting(dion, _), _)), 
   assert(dig_db_element(setting(dion, kb), kb(dion,default))),
   assert(pion_log_counter(0)),
   assert(dig_db_element(setting(dion, pinpointDirectory), 'pinpoint')),
   assert(dig_db_element(setting(dion, pinpointFileDirectory), 'pinpoint2')),
   set_dion_local_do_predicate,
   dig_set_server_port(dion, 8004),
   true.


set_dion_local_do_predicate:-
    assert(dig_db_element(setting(dion, localDoPredicates), [get_mups, get_pinpoints_from_file, get_pinpoints_from_ontology, get_pinpoints_from_kbid, test])).


dion_do_processing(get_mups, P, [A]):-
     select(element(concept, [value=C], []), P, P1),
     select(element(kbid, [value=KBID1], []),P1, _),
    (atom(KBID1)->atom_to_term(KBID1, KBID,_); KBID=KBID1),
     !,
     get_mups(KBID, C, ML),
     length(ML, N),
     mupss_mups(ML, ME1),
     A = element('MUPSs', [concept=C, number=N], ME1),
     true.

dion_do_processing(get_pinpoints_from_file, Q, [A]):-
     select(element(ontology, [value=O], []), Q, _Q1),
     !,
     get_pinpoints_from_file(O, P),
     length(P, N),
     pinpoints_formatted(P, P1),
     A = element('pinpoints', [ontology=O, number=N], P1),
     true.

dion_do_processing(get_pinpoints_from_ontology, Q, [A]):-
     select(element(ontology, [value=O], []), Q, _Q1),
     !,
     get_pinpoints_from_ontology(O, P),
     length(P, N),
     pinpoints_formatted(P, P1),
     A = element('pinpoints', [ontology=O, number=N], P1),
     true.

dion_do_processing(get_pinpoints_from_kbid, Q, [A]):-
    select(element(kbid, [value=KBID1], []),Q, _),
   (atom(KBID1)->atom_to_term(KBID1, KBID,_); KBID=KBID1),
     !,
     get_pinpoints_from_kbid(KBID, P),
     length(P, N),
     pinpoints_formatted(P, P1),
     KBID = kb(dion, O),
     A = element('pinpoints', [ontology=O, number=N], P1),
     true.



dion_do_processing(test, _Parameters, [A]):-
    A = element(ok, [do = 'test', kbid = kb(dion)], []),
    true.


dion_do_predicate(P, _Ps, [A]):-
      A = element(error, [reason='unsupported do_function or wrong parameters', function=P], []).


	
dig_get_server_port(System, Port):-
  dig_db_element(setting(System, serverPort), Port).

dig_set_server_port(System, Port):-
  retractall(dig_db_element(setting(System, serverPort), _)), 
  assert(dig_db_element(setting(System, serverPort), Port)). 

 




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% predicates for debugging and log only
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



message_log(_L):-
      log_counter(dion, I),
	I > 200,  % limit of the log counter
      !.

message_log(L):-
      log_counter(dion, I),
      I1 is I+1,
      retract(log_counter(dion, I)),
      assert(log_counter(dion, I1)),
      concat_atom(['log/tmp_tell_log', I, '.xml'], File0),
      !,
      open(File0, write, Out, []),
	dig_xmlns(XMLNS),
	TellElements = [element(tells, XMLNS, L)],
      elements_xmltext(TellElements, file(File0)),
 	close(Out),
      true.

message_log(_L):-!.

%/*

% a test to create unsatisfiable concept on a file
testit2:- 	
   %        Ontology ='example-t1.dig',
            Ontology = 'MGEDinconsist',
		KB = kb(dion, Ontology),
            unsatisfiable_concept_list_to_file(Ontology, KB, _L, CL, N, _UCText),         
            format('~n~w UNSATISFIABLE CONCEPTs:~w~n~n', [N, CL]),
            true.


% a test to create unsatisfiable concept on a file and get mups

testit21:- 	KB = kb(dion, Ontology),
 %           Ontology ='example-t1.dig',
           Ontology = 'MGEDinconsist',
            unsatisfiable_concept_list_to_file(Ontology, KB, L, CL, N, _UCText),         
            format('~n~w UNSATISFIABLE CONCEPTs:~w~n~n', [N, CL]),           
		mups(KB, L, ML),            
            format('MUPS:~w~n~n', [ML]),           
            true.


% a test to get mups for a concept and load an ontology

testit22:- C='Action',
           testit22(C).

testit22(C):-
		Ontology = 'MGEDinconsist',
 %          Ontology ='example-t1.dig',
		KB = kb(dion, Ontology),
		concat_atom([Ontology, '.xml'], File),           
            dig_load_telldata(File, KB),         
		get_mups(KB, C, ML),
            length(ML, N),            
            format('~w MUPSs for concept ~w~n~n', [N, C]), 
    		mups_log(C, ML),          
            true.

% a test to get mups for a concept

testit23:- C='Action',
           testit23(C).

testit23(C):- 	
            Ontology = 'MGEDinconsist',
 %          Ontology ='example-t1.dig',
		KB = kb(dion, Ontology),       
		get_mups(KB, C, ML),
            length(ML, N),            
            format('~w MUPSs for concept ~w~n~n', [N, C]),
            mups_log(C, ML),           
            true.


% a test to create unsatisfiable concept, mups, mips, core, and pointpoint on a file

testit5:- 	
            Ontology ='example-t1.dig',
 %          Ontology = 'MGEDinconsist',
            KB = kb(dion, Ontology),
            unsatisfiable_concept_list_to_file(Ontology, KB, L, CL, N, _UCText),
            format('~n~w UNSATISFIABLE CONCEPTs:~w~n~n', [N, CL]),           
            mups(KB, L, ML),
            mups_mips_core_pinpoint(Ontology, ML, P),
            format('pinpoint:~w~n', [P]),
            true.



% a test to get unsatisfiable concept, mups, mips, core, and pointpoint without writing on the files

testit6:- 	
		Ontology ='example-t1.dig',
 %          Ontology = 'MGEDinconsist',
            KB = kb(dion,Ontology),
 %          assert(log_counter(dion, 0)),
		unsatisfiable_concept_list_to_file(Ontology, KB, L, CL, N, _UCText),
            format('~n~w UNSATISFIABLE CONCEPTs:~w~n~n', [N, CL]),                       
            mups(KB, L, ML),
            mips_axiom(ML, ML1),
            format('MUPS(elements):~w~n~n', [ML]),
            format('MUPS(axiom):~w~n~n', [ML1]),
            mips(ML, M),
            mips_axiom(M, M1),
            format('MIPS:~w~n~n', [M1]),
            mips_core(M1, C),
            format('CORE: ~w~n~n', [C]),
            pinpoints(C, M1, P),
            format('PINPOINTS: ~w~n~n', [P]),
            true.

% the same like testit6, but the old format and predicates

testit61:- 	KB = kb(dion),
            assert(log_counter(dion, 0)),
		dig_load_telldata('example-t1.dig.xml', KB),
%		dig_load_telldata('MGEDinconsist.xml', KB),
            dig_unsatisfiable_conceptname(KB, L),
            pion_icr_set(L, icr(_, CL, _)),
            format('~n~nUNSATISFIABLE CONCEPT:~w~n~n', [CL]),
            mups(KB, L, ML),
            mips_axiom(ML, ML1),
            format('MUPS:~w~n~n', [ML1]),
            mips(ML, M),
            mips_axiom(M, M1),
            format('MIPS:~w~n~n', [M1]),
            mips_core(M1, C),
            format('CORE: ~w~n~n', [C]),
            pinpoints(C, M1, P),
            format('PINPOINTS: ~w~n~n', [P]),
            true.




% a test to read from a file of unsatisfiable concept and files from mups, 
% then creating mips, core, and pointpoint on files

testit51:-  
%           Ontology ='example-t1.dig',
            Ontology = 'MGEDinconsist',
 		get_unsatisfiable_concept_from_file(Ontology, L, N),
            format('Unsatisfiable ~w Concepts:~w~n', [N, L]),
            get_mupss_from_file(L, ML),           
 %           format('get mupss ~w~n', [ML]),
            mups_mips_core_pinpoint(Ontology, ML, P),
            format('pinpoint:~w~n', [P]),
            true.



testit52:-
		C = 'Action',
%		C = 'a1',
        	get_mups_from_file(C, L, N),
            mips_axiom(L, L1),
            format('~w MUPS for Concepts ~w: ~w~n', [N, C, L1]),
            true.
%*/

%:- dion_init.

:- dion_init, dig_get_server_port(dion, Port), set_dig_server_id('SEKT DION Server 0.8.1'), http_server(dig_server,[port(Port)]).




